@@ -1,4 +1,4 @@
-from typing import Optional, Dict, List, Union
+from typing import Generator, Optional, Dict, List, Union
 
 import logging
 import collections
@@ -7,6 +7,11 @@ from itertools import islice
 from abc import abstractmethod
 from pathlib import Path
 
+try:
+    from typing import Literal
+except ImportError:
+    from typing_extensions import Literal #type: ignore
+
 from haystack.schema import Document, Label, MultiLabel
 from haystack.nodes.base import BaseComponent
 from haystack.errors import DuplicateDocumentError
@@ -34,12 +39,12 @@ class BaseKnowledgeGraph(BaseComponent):
     """
     outgoing_edges = 1
 
-    def run(self, sparql_query: str, index: Optional[str] = None, **kwargs):  # type: ignore
-        result = self.query(sparql_query=sparql_query, index=index)
+    def run(self, sparql_query: str, index: Optional[str] = None, headers: Optional[Dict[str, str]] = None):  # type: ignore
+        result = self.query(sparql_query=sparql_query, index=index, headers=headers)
         output = {"sparql_result": result}
         return output, "output_1"
 
-    def query(self, sparql_query: str, index: Optional[str] = None):
+    def query(self, sparql_query: str, index: Optional[str] = None, headers: Optional[Dict[str, str]] = None):
         raise NotImplementedError
 
 
@@ -55,7 +60,8 @@ class BaseDocumentStore(BaseComponent):
 
     @abstractmethod
     def write_documents(self, documents: Union[List[dict], List[Document]], index: Optional[str] = None,
-                        batch_size: int = 10_000, duplicate_documents: Optional[str] = None):
+                        batch_size: int = 10_000, duplicate_documents: Optional[str] = None, 
+                        headers: Optional[Dict[str, str]] = None):
         """
         Indexes documents for later queries.
 
@@ -73,6 +79,7 @@ class BaseDocumentStore(BaseComponent):
                                     overwrite: Update any existing documents with the same ID when adding documents.
                                     fail: an error is raised if the document ID of the document being added already
                                     exists.
+        :param headers: Custom HTTP headers to pass to document store client if supported (e.g. {'Authorization': 'Basic YWRtaW46cm9vdA=='} for basic authentication)
 
         :return: None
         """
@@ -83,7 +90,9 @@ class BaseDocumentStore(BaseComponent):
             self,
             index: Optional[str] = None,
             filters: Optional[Dict[str, List[str]]] = None,
-            return_embedding: Optional[bool] = None
+            return_embedding: Optional[bool] = None, 
+            batch_size: int = 10_000, 
+            headers: Optional[Dict[str, str]] = None
     ) -> List[Document]:
         """
         Get documents from the document store.
@@ -93,6 +102,32 @@ class BaseDocumentStore(BaseComponent):
         :param filters: Optional filters to narrow down the documents to return.
                         Example: {"name": ["some", "more"], "category": ["only_one"]}
         :param return_embedding: Whether to return the document embeddings.
+        :param batch_size: Number of documents that are passed to bulk function at a time.
+        :param headers: Custom HTTP headers to pass to document store client if supported (e.g. {'Authorization': 'Basic YWRtaW46cm9vdA=='} for basic authentication)
+        """
+        pass
+
+    @abstractmethod
+    def get_all_documents_generator(
+        self,
+        index: Optional[str] = None,
+        filters: Optional[Dict[str, List[str]]] = None,
+        return_embedding: Optional[bool] = None,
+        batch_size: int = 10_000, 
+        headers: Optional[Dict[str, str]] = None
+    ) -> Generator[Document, None, None]:
+        """
+        Get documents from the document store. Under-the-hood, documents are fetched in batches from the
+        document store and yielded as individual documents. This method can be used to iteratively process
+        a large number of documents without having to load all documents in memory.
+
+        :param index: Name of the index to get the documents from. If None, the
+                      DocumentStore's default index (self.index) will be used.
+        :param filters: Optional filters to narrow down the documents to return.
+                        Example: {"name": ["some", "more"], "category": ["only_one"]}
+        :param return_embedding: Whether to return the document embeddings.
+        :param batch_size: When working with large number of documents, batching can help reduce memory footprint.
+        :param headers: Custom HTTP headers to pass to document store client if supported (e.g. {'Authorization': 'Basic YWRtaW46cm9vdA=='} for basic authentication)
         """
         pass
 
@@ -111,7 +146,7 @@ class BaseDocumentStore(BaseComponent):
             return ret
 
     @abstractmethod
-    def get_all_labels(self, index: Optional[str] = None, filters: Optional[Dict[str, List[str]]] = None) -> List[Label]:
+    def get_all_labels(self, index: Optional[str] = None, filters: Optional[Dict[str, List[str]]] = None, headers: Optional[Dict[str, str]] = None) -> List[Label]:
         pass
 
     def get_all_labels_aggregated(self,
@@ -120,7 +155,8 @@ class BaseDocumentStore(BaseComponent):
                                   open_domain: bool=True,
                                   drop_negative_labels: bool=False,
                                   drop_no_answers: bool=False,
-                                  aggregate_by_meta: Optional[Union[str, list]]=None) -> List[MultiLabel]:
+                                  aggregate_by_meta: Optional[Union[str, list]]=None,
+                                  headers: Optional[Dict[str, str]] = None) -> List[MultiLabel]:
         """
         Return all labels in the DocumentStore, aggregated into MultiLabel objects. 
         This aggregation step helps, for example, if you collected multiple possible answers for one question and you
@@ -142,12 +178,13 @@ class BaseDocumentStore(BaseComponent):
                             When False, labels are aggregated in a closed domain fashion based on the question text
                             and also the id of the document that the label is tied to. In this setting, this function
                             might return multiple MultiLabel objects with the same question string.
+        :param headers: Custom HTTP headers to pass to document store client if supported (e.g. {'Authorization': 'Basic YWRtaW46cm9vdA=='} for basic authentication)
         :param TODO drop params
         :param aggregate_by_meta: The names of the Label meta fields by which to aggregate. For example: ["product_id"]
 
         """
         aggregated_labels = []
-        all_labels = self.get_all_labels(index=index, filters=filters)
+        all_labels = self.get_all_labels(index=index, filters=filters, headers=headers)
 
         # Collect all answers to a question in a dict
         question_ans_dict: dict = {}
@@ -184,29 +221,38 @@ class BaseDocumentStore(BaseComponent):
         return aggregated_labels
 
     @abstractmethod
-    def get_document_by_id(self, id: str, index: Optional[str] = None) -> Optional[Document]:
+    def get_document_by_id(self, id: str, index: Optional[str] = None, headers: Optional[Dict[str, str]] = None) -> Optional[Document]:
         pass
 
     @abstractmethod
-    def get_document_count(self, filters: Optional[Dict[str, List[str]]] = None, index: Optional[str] = None) -> int:
+    def get_document_count(self, 
+        filters: Optional[Dict[str, List[str]]] = None, 
+        index: Optional[str] = None,
+        only_documents_without_embedding: bool = False, 
+        headers: Optional[Dict[str, str]] = None) -> int:
         pass
 
+    @staticmethod
     @njit#(fastmath=True)
-    def normalize_embedding(self, emb: np.ndarray) -> None:
+    def normalize_embedding(emb: np.ndarray) -> None:
         """
-        Performs L2 normalization of embeddings vector inplace. Input can be a single vector (1D array) or a matrix (2D array).
+        Performs L2 normalization of embeddings vector inplace. Input can be a single vector (1D array) or a matrix
+        (2D array).
         """
         # Might be extended to other normalizations in future
 
         # Single vec
         if len(emb.shape) == 1:
-            norm = np.sqrt(emb.dot(emb)) #faster than np.linalg.norm()
+            norm = np.sqrt(emb.dot(emb))  # faster than np.linalg.norm()
             if norm != 0.0:
                 emb /= norm
         # 2D matrix
         else:
-            norm = np.linalg.norm(emb, axis=1)
-            emb /= norm[:, None]
+            for vec in emb:
+                vec = np.ascontiguousarray(vec)
+                norm = np.sqrt(vec.dot(vec))
+                if norm != 0.0:
+                    vec /= norm
 
     def finalize_raw_score(self, raw_score: float, similarity: Optional[str]) -> float:
         if similarity == "cosine":
@@ -220,20 +266,22 @@ class BaseDocumentStore(BaseComponent):
                            filters: Optional[Optional[Dict[str, List[str]]]] = None,
                            top_k: int = 10,
                            index: Optional[str] = None,
-                           return_embedding: Optional[bool] = None) -> List[Document]:
+                           return_embedding: Optional[bool] = None,
+                           headers: Optional[Dict[str, str]] = None) -> List[Document]:
         pass
 
     @abstractmethod
-    def get_label_count(self, index: Optional[str] = None) -> int:
+    def get_label_count(self, index: Optional[str] = None, headers: Optional[Dict[str, str]] = None) -> int:
         pass
 
     @abstractmethod
-    def write_labels(self, labels: Union[List[Label], List[dict]], index: Optional[str] = None):
+    def write_labels(self, labels: Union[List[Label], List[dict]], index: Optional[str] = None, headers: Optional[Dict[str, str]] = None):
         pass
 
     def add_eval_data(self, filename: str, doc_index: str = "eval_document", label_index: str = "label",
                       batch_size: Optional[int] = None, preprocessor: Optional[PreProcessor] = None,
-                      max_docs: Union[int, bool] = None, open_domain: bool = False):
+                      max_docs: Union[int, bool] = None, open_domain: bool = False, 
+                      headers: Optional[Dict[str, str]] = None):
         """
         Adds a SQuAD-formatted file to the DocumentStore in order to be able to perform evaluation on it.
         If a jsonl file and a batch_size is passed to the function, documents are loaded batchwise
@@ -252,6 +300,7 @@ class BaseDocumentStore(BaseComponent):
                          When set to None (default) all available eval documents are used.
         :param open_domain: Set this to True if your file is an open domain dataset where two different answers to the
                             same question might be found in different contexts.
+        :param headers: Custom HTTP headers to pass to document store client if supported (e.g. {'Authorization': 'Basic YWRtaW46cm9vdA=='} for basic authentication)
 
         """
         # TODO improve support for PreProcessor when adding eval data
@@ -274,43 +323,63 @@ class BaseDocumentStore(BaseComponent):
         if file_path.suffix == ".json":
             if batch_size is None:
                 docs, labels = eval_data_from_json(filename, max_docs=max_docs, preprocessor=preprocessor, open_domain=open_domain)
-                self.write_documents(docs, index=doc_index)
-                self.write_labels(labels, index=label_index)
+                self.write_documents(docs, index=doc_index, headers=headers)
+                self.write_labels(labels, index=label_index, headers=headers)
             else:
                 jsonl_filename = (file_path.parent / (file_path.stem + '.jsonl')).as_posix()
                 logger.info(f"Adding evaluation data batch-wise is not compatible with json-formatted SQuAD files. "
                             f"Converting json to jsonl to: {jsonl_filename}")
                 squad_json_to_jsonl(filename, jsonl_filename)
-                self.add_eval_data(jsonl_filename, doc_index, label_index, batch_size, open_domain=open_domain)
+                self.add_eval_data(jsonl_filename, doc_index, label_index, batch_size, open_domain=open_domain, headers=headers)
 
         elif file_path.suffix == ".jsonl":
             for docs, labels in eval_data_from_jsonl(filename, batch_size, max_docs=max_docs, preprocessor=preprocessor, open_domain=open_domain):
                 if docs:
-                    self.write_documents(docs, index=doc_index)
+                    self.write_documents(docs, index=doc_index, headers=headers)
                 if labels:
-                    self.write_labels(labels, index=label_index)
+                    self.write_labels(labels, index=label_index, headers=headers)
 
         else:
             logger.error("File needs to be in json or jsonl format.")
 
-    def delete_all_documents(self, index: Optional[str] = None, filters: Optional[Dict[str, List[str]]] = None):
+    def delete_all_documents(self, index: Optional[str] = None, filters: Optional[Dict[str, List[str]]] = None, headers: Optional[Dict[str, str]] = None):
         pass
 
     @abstractmethod
-    def delete_documents(self, index: Optional[str] = None, ids: Optional[List[str]] = None, filters: Optional[Dict[str, List[str]]] = None):
+    def delete_documents(self, index: Optional[str] = None, ids: Optional[List[str]] = None, filters: Optional[Dict[str, List[str]]] = None, headers: Optional[Dict[str, str]] = None):
         pass
 
     @abstractmethod
-    def delete_labels(self, index: Optional[str] = None, ids: Optional[List[str]] = None, filters: Optional[Dict[str, List[str]]] = None):
+    def delete_labels(self, index: Optional[str] = None, ids: Optional[List[str]] = None, filters: Optional[Dict[str, List[str]]] = None, headers: Optional[Dict[str, str]] = None):
+        pass
+    
+    @abstractmethod
+    def _create_document_field_map(self) -> Dict:
         pass
 
-    def run(self, documents: List[dict], index: Optional[str] = None):  # type: ignore
-        self.write_documents(documents=documents, index=index)
+    def run(self, documents: List[dict], index: Optional[str] = None, headers: Optional[Dict[str, str]] = None, id_hash_keys: Optional[List[str]] = None  ):  # type: ignore
+        """
+        Run requests of document stores 
+
+        Comment: We will gradually introduce the primitives. The doument stores also accept dicts and parse them to documents. 
+        In the future, however, only documents themselves will be accepted. Parsing the dictionaries in the run function 
+        is therefore only an interim solution until the run function also accepts documents. 
+
+        :param documents: A list of dicts that are documents.
+        :param headers: A list of headers.
+        :param index: Optional name of index where the documents shall be written to.
+                      If None, the DocumentStore's default index (self.index) will be used.
+        :param id_hash_keys: List of the fields that the hashes of the ids are generated from.
+        """
+        
+        field_map = self._create_document_field_map()
+        doc_objects = [Document.from_dict(d, field_map=field_map, id_hash_keys=id_hash_keys) for d in documents]
+        self.write_documents(documents=doc_objects, index=index, headers=headers)
         return {}, "output_1"
 
     @abstractmethod
     def get_documents_by_id(self, ids: List[str], index: Optional[str] = None,
-                            batch_size: int = 10_000) -> List[Document]:
+                            batch_size: int = 10_000, headers: Optional[Dict[str, str]] = None) -> List[Document]:
         pass
 
     def _drop_duplicate_documents(self, documents: List[Document]) -> List[Document]:
@@ -336,7 +405,8 @@ class BaseDocumentStore(BaseComponent):
     def _handle_duplicate_documents(self,
                                     documents: List[Document],
                                     index: Optional[str] = None,
-                                    duplicate_documents: Optional[str] = None):
+                                    duplicate_documents: Optional[str] = None,
+                                    headers: Optional[Dict[str, str]] = None):
         """
         Checks whether any of the passed documents is already existing in the chosen index and returns a list of
         documents that are not in the index yet.
@@ -348,13 +418,14 @@ class BaseDocumentStore(BaseComponent):
                                     overwrite: Update any existing documents with the same ID when adding documents.
                                     fail: an error is raised if the document ID of the document being added already
                                     exists.
+        :param headers: Custom HTTP headers to pass to document store client if supported (e.g. {'Authorization': 'Basic YWRtaW46cm9vdA=='} for basic authentication)
         :return: A list of Haystack Document objects.
        """
 
         index = index or self.index
         if duplicate_documents in ('skip', 'fail'):
             documents = self._drop_duplicate_documents(documents)
-            documents_found = self.get_documents_by_id(ids=[doc.id for doc in documents], index=index)
+            documents_found = self.get_documents_by_id(ids=[doc.id for doc in documents], index=index, headers=headers)
             ids_exist_in_db: List[str] = [doc.id for doc in documents_found]
 
             if len(ids_exist_in_db) > 0 and duplicate_documents == 'fail':
@@ -365,11 +436,12 @@ class BaseDocumentStore(BaseComponent):
 
         return documents
 
-    def _get_duplicate_labels(self, labels: list, index: str = None) -> List[Label]:
+    def _get_duplicate_labels(self, labels: list, index: str = None, headers: Optional[Dict[str, str]] = None) -> List[Label]:
         """
         Return all duplicate labels
         :param labels: List of Label objects
         :param index: add an optional index attribute to labels. It can be later used for filtering.
+        :param headers: Custom HTTP headers to pass to document store client if supported (e.g. {'Authorization': 'Basic YWRtaW46cm9vdA=='} for basic authentication)
         :return: List of labels
         """
         index = index or self.label_index
@@ -380,7 +452,7 @@ class BaseDocumentStore(BaseComponent):
             if count > 1:
                 duplicate_ids.append(label_id)
 
-        for label in self.get_all_labels(index=index):
+        for label in self.get_all_labels(index=index, headers=headers):
             if label.id in new_ids:
                 duplicate_ids.append(label.id)
 
