@@ -70,8 +70,9 @@ class InMemoryDocumentStore(BaseDocumentStore):
         self.progress_bar = progress_bar
         self.duplicate_documents = duplicate_documents
 
-    def write_documents(self, documents: Union[List[dict], List[Document]], index: Optional[str] = None,  # type: ignore
-                        duplicate_documents: Optional[str] = None):
+    def write_documents(self, documents: Union[List[dict], List[Document]], index: Optional[str] = None,
+                        batch_size: int = 10_000, duplicate_documents: Optional[str] = None, 
+                        headers: Optional[Dict[str, str]] = None):
         """
         Indexes documents for later queries.
 
@@ -92,6 +93,9 @@ class InMemoryDocumentStore(BaseDocumentStore):
         :raises DuplicateDocumentError: Exception trigger on duplicate document
         :return: None
         """
+        if headers:
+            raise NotImplementedError("InMemoryDocumentStore does not support headers.")
+
         index = index or self.index
         duplicate_documents = duplicate_documents or self.duplicate_documents
         assert duplicate_documents in self.duplicate_documents_options, \
@@ -118,10 +122,13 @@ class InMemoryDocumentStore(BaseDocumentStore):
             self.embedding_field: "embedding",
         }
 
-    def write_labels(self, labels: Union[List[dict], List[Label]], index: Optional[str] = None):
+    def write_labels(self, labels: Union[List[dict], List[Label]], index: Optional[str] = None, headers: Optional[Dict[str, str]] = None):
         """
         Write annotation labels into document store.
         """
+        if headers:
+            raise NotImplementedError("InMemoryDocumentStore does not support headers.")
+        
         index = index or self.label_index
         label_objects = [Label.from_dict(l) if isinstance(l, dict) else l for l in labels]
 
@@ -140,10 +147,13 @@ class InMemoryDocumentStore(BaseDocumentStore):
                 label.updated_at = label.created_at
             self.indexes[index][label.id] = label
 
-    def get_document_by_id(self, id: str, index: Optional[str] = None) -> Optional[Document]:
+    def get_document_by_id(self, id: str, index: Optional[str] = None, headers: Optional[Dict[str, str]] = None) -> Optional[Document]:
         """
         Fetch a document by specifying its text id string.
         """
+        if headers:
+            raise NotImplementedError("InMemoryDocumentStore does not support headers.")
+        
         index = index or self.index
         documents = self.get_documents_by_id([id], index=index)
         if documents:
@@ -164,7 +174,8 @@ class InMemoryDocumentStore(BaseDocumentStore):
                            filters: Optional[Dict[str, List[str]]] = None,
                            top_k: int = 10,
                            index: Optional[str] = None,
-                           return_embedding: Optional[bool] = None) -> List[Document]:
+                           return_embedding: Optional[bool] = None,
+                           headers: Optional[Dict[str, str]] = None) -> List[Document]:
         """
         Find the document that is most similar to the provided `query_emb` by using a vector similarity metric.
 
@@ -176,6 +187,9 @@ class InMemoryDocumentStore(BaseDocumentStore):
         :param return_embedding: To return document embedding
         :return:
         """
+        if headers:
+            raise NotImplementedError("InMemoryDocumentStore does not support headers.")
+        
         index = index or self.index
         if return_embedding is None:
             return_embedding = self.return_embedding
@@ -196,13 +210,11 @@ class InMemoryDocumentStore(BaseDocumentStore):
             new_document.embedding = doc.embedding if return_embedding is True else None
 
             if self.similarity == "dot_product":
-                score = np.dot(query_emb, doc.embedding) / (
-                    np.linalg.norm(query_emb) * np.linalg.norm(doc.embedding)
-                )
+                score = np.dot(query_emb, doc.embedding)
             elif self.similarity == "cosine":
                 # cosine similarity score = 1 - cosine distance
                 score = 1 - cosine(query_emb, doc.embedding)
-            new_document.score = (score + 1) / 2
+            new_document.score = self.finalize_raw_score(score, self.similarity)
             candidate_docs.append(new_document)
 
         return sorted(candidate_docs, key=lambda x: x.score if x.score is not None else 0.0, reverse=True)[0:top_k]
@@ -259,11 +271,14 @@ class InMemoryDocumentStore(BaseDocumentStore):
                 progress_bar.set_description_str("Documents Processed")
                 progress_bar.update(batch_size)
 
-    def get_document_count(self, filters: Optional[Dict[str, List[str]]] = None, index: Optional[str] = None) -> int:
+    def get_document_count(self, filters: Optional[Dict[str, List[str]]] = None, index: Optional[str] = None, only_documents_without_embedding: bool = False, headers: Optional[Dict[str, str]] = None) -> int:
         """
         Return the number of documents in the document store.
         """
-        documents = self.get_all_documents(index=index, filters=filters)
+        if headers:
+            raise NotImplementedError("InMemoryDocumentStore does not support headers.")
+        
+        documents = self._query(index=index, filters=filters, only_documents_without_embedding=only_documents_without_embedding)
         return len(documents)
 
     def get_embedding_count(self, filters: Optional[Dict[str, List[str]]] = None, index: Optional[str] = None) -> int:
@@ -274,10 +289,13 @@ class InMemoryDocumentStore(BaseDocumentStore):
         embedding_count = sum(doc.embedding is not None for doc in documents)
         return embedding_count
 
-    def get_label_count(self, index: Optional[str] = None) -> int:
+    def get_label_count(self, index: Optional[str] = None, headers: Optional[Dict[str, str]] = None) -> int:
         """
         Return the number of labels in the document store.
         """
+        if headers:
+            raise NotImplementedError("InMemoryDocumentStore does not support headers.")
+        
         index = index or self.label_index
         return len(self.indexes[index].items())
 
@@ -286,8 +304,7 @@ class InMemoryDocumentStore(BaseDocumentStore):
         index: Optional[str] = None,
         filters: Optional[Dict[str, List[str]]] = None,
         return_embedding: Optional[bool] = None,
-        only_documents_without_embedding: bool = False,
-        batch_size: int = 10_000,
+        only_documents_without_embedding: bool = False
     ):
         index = index or self.index
         documents = deepcopy(list(self.indexes[index].values()))
@@ -325,6 +342,7 @@ class InMemoryDocumentStore(BaseDocumentStore):
         filters: Optional[Dict[str, List[str]]] = None,
         return_embedding: Optional[bool] = None,
         batch_size: int = 10_000,
+        headers: Optional[Dict[str, str]] = None
     ) -> List[Document]:
         """
         Get all documents from the document store as a list.
@@ -335,7 +353,10 @@ class InMemoryDocumentStore(BaseDocumentStore):
                         Example: {"name": ["some", "more"], "category": ["only_one"]}
         :param return_embedding: Whether to return the document embeddings.
         """
-        result = self.get_all_documents_generator(index=index, filters=filters, return_embedding=return_embedding)
+        if headers:
+            raise NotImplementedError("InMemoryDocumentStore does not support headers.")
+        
+        result = self.get_all_documents_generator(index=index, filters=filters, return_embedding=return_embedding, batch_size=batch_size)
         documents = list(result)
         return documents
 
@@ -345,6 +366,7 @@ class InMemoryDocumentStore(BaseDocumentStore):
         filters: Optional[Dict[str, List[str]]] = None,
         return_embedding: Optional[bool] = None,
         batch_size: int = 10_000,
+        headers: Optional[Dict[str, str]] = None
     ) -> Generator[Document, None, None]:
         """
         Get all documents from the document store. The methods returns a Python Generator that yields individual
@@ -356,18 +378,23 @@ class InMemoryDocumentStore(BaseDocumentStore):
                         Example: {"name": ["some", "more"], "category": ["only_one"]}
         :param return_embedding: Whether to return the document embeddings.
         """
+        if headers:
+            raise NotImplementedError("InMemoryDocumentStore does not support headers.")
+        
         result = self._query(
             index=index,
             filters=filters,
-            return_embedding=return_embedding,
-            batch_size=batch_size
+            return_embedding=return_embedding
         )
         yield from result
 
-    def get_all_labels(self, index: str = None, filters: Optional[Dict[str, List[str]]] = None) -> List[Label]:
+    def get_all_labels(self, index: str = None, filters: Optional[Dict[str, List[str]]] = None, headers: Optional[Dict[str, str]] = None) -> List[Label]:
         """
         Return all labels in the document store.
         """
+        if headers:
+            raise NotImplementedError("InMemoryDocumentStore does not support headers.")
+        
         index = index or self.label_index
 
         if filters:
@@ -386,7 +413,7 @@ class InMemoryDocumentStore(BaseDocumentStore):
 
         return result
 
-    def delete_all_documents(self, index: Optional[str] = None, filters: Optional[Dict[str, List[str]]] = None):
+    def delete_all_documents(self, index: Optional[str] = None, filters: Optional[Dict[str, List[str]]] = None, headers: Optional[Dict[str, str]] = None):
         """
         Delete documents in an index. All documents are deleted if no filters are passed.
 
@@ -394,6 +421,9 @@ class InMemoryDocumentStore(BaseDocumentStore):
         :param filters: Optional filters to narrow down the documents to be deleted.
         :return: None
         """
+        if headers:
+            raise NotImplementedError("InMemoryDocumentStore does not support headers.")
+        
         logger.warning(
                 """DEPRECATION WARNINGS: 
                 1. delete_all_documents() method is deprecated, please use delete_documents method
@@ -402,7 +432,7 @@ class InMemoryDocumentStore(BaseDocumentStore):
         )
         self.delete_documents(index, None, filters)
 
-    def delete_documents(self, index: Optional[str] = None, ids: Optional[List[str]] = None, filters: Optional[Dict[str, List[str]]] = None):
+    def delete_documents(self, index: Optional[str] = None, ids: Optional[List[str]] = None, filters: Optional[Dict[str, List[str]]] = None, headers: Optional[Dict[str, str]] = None):
         """
         Delete documents in an index. All documents are deleted if no filters are passed.
 
@@ -417,6 +447,9 @@ class InMemoryDocumentStore(BaseDocumentStore):
 
         :return: None
         """
+        if headers:
+            raise NotImplementedError("InMemoryDocumentStore does not support headers.")
+        
         index = index or self.index
         if not filters and not ids:
             self.indexes[index] = {}
@@ -427,7 +460,7 @@ class InMemoryDocumentStore(BaseDocumentStore):
         for doc in docs_to_delete:
             del self.indexes[index][doc.id]
 
-    def delete_labels(self, index: Optional[str] = None, ids: Optional[List[str]] = None, filters: Optional[Dict[str, List[str]]] = None):
+    def delete_labels(self, index: Optional[str] = None, ids: Optional[List[str]] = None, filters: Optional[Dict[str, List[str]]] = None, headers: Optional[Dict[str, str]] = None):
         """
         Delete labels in an index. All labels are deleted if no filters are passed.
 
@@ -438,6 +471,9 @@ class InMemoryDocumentStore(BaseDocumentStore):
                         Example filters: {"id": ["9a196e41-f7b5-45b4-bd19-5feb7501c159", "9a196e41-f7b5-45b4-bd19-5feb7501c159"]} or {"query": ["question2"]}
         :return: None
         """
+        if headers:
+            raise NotImplementedError("InMemoryDocumentStore does not support headers.")
+        
         index = index or self.label_index
         if not filters and not ids:
             self.indexes[index] = {}
