@@ -368,7 +368,8 @@ class Pipeline(BasePipeline):
         self,
         labels: List[MultiLabel],
         params: Optional[dict] = None,
-        sas_model_name_or_path: str = None
+        sas_model_name_or_path: str = None,
+        add_isolated_node_eval: bool = False
     ) -> EvaluationResult:
         """
             Evaluates the pipeline by running the pipeline once per query in debug mode 
@@ -390,8 +391,20 @@ class Pipeline(BasePipeline):
                         - Good default for multiple languages: "sentence-transformers/paraphrase-multilingual-mpnet-base-v2"
                         - Large, powerful, but slow model for English only: "cross-encoder/stsb-roberta-large"
                         - Large model for German only: "deepset/gbert-large-sts"
+            :param add_isolated_node_eval: If set to True, in addition to the integrated evaluation of the pipeline, each node is evaluated in isolated evaluation mode.
+                        This mode helps to understand the bottlenecks of a pipeline in terms of output quality of each individual node.
+                        If a node performs much better in the isolated evaluation than in the integrated evaluation, the previous node needs to be optimized to improve the pipeline's performance.
+                        If a node's performance is similar in both modes, this node itself needs to be optimized to improve the pipeline's performance.
+                        The isolated evaluation calculates the upper bound of each node's evaluation metrics under the assumption that it received perfect inputs from the previous node.
+                        To this end, labels are used as input to the node instead of the output of the previous node in the pipeline.
+                        The generated dataframes in the EvaluationResult then contain additional rows, which can be distinguished from the integrated evaluation results based on the
+                        values "integrated" or "isolated" in the column "eval_mode" and the evaluation report then additionally lists the upper bound of each node's evaluation metrics.
         """    
         eval_result = EvaluationResult()
+        if add_isolated_node_eval:
+            if params is None:
+                params = {}
+            params["add_isolated_node_eval"] = True
         queries = [label.query for label in labels]
         for query, label in zip(queries, labels):
             predictions = self.run(query=query, labels=label, params=params, debug=True)
@@ -413,8 +426,25 @@ class Pipeline(BasePipeline):
                                                 sas_model_name_or_path=sas_model_name_or_path)
                     df["sas"] = sas
 
+        # reorder columns for better qualitative evaluation
+        for key, df in eval_result.node_results.items():
+            desired_col_order = ["query", # generic
+                                "gold_answers", "answer", "context", "exact_match", "f1", "sas", # answer-specific
+                                "gold_document_contents", "content", "gold_id_match", "answer_match", "gold_id_or_answer_match", # doc-specific
+                                "rank", "document_id", "gold_document_ids", # generic
+                                "offsets_in_document", "gold_offsets_in_documents", # answer-specific
+                                "type", "node", "eval_mode"] # generic
+            eval_result.node_results[key] = self._reorder_columns(df, desired_col_order)
+
         return eval_result
 
+    def _reorder_columns(self, df: DataFrame, desired_order: List[str]) -> DataFrame:
+        filtered_order = [col for col in desired_order if col in df.columns]
+        missing_columns = [col for col in df.columns if col not in desired_order]
+        reordered_columns = filtered_order + missing_columns
+        assert len(reordered_columns) == len(df.columns)
+        return df.reindex(columns=reordered_columns)
+
     def _build_eval_dataframe(self, 
         query: str, 
         query_labels: MultiLabel, 
@@ -430,11 +460,10 @@ class Pipeline(BasePipeline):
         Additional answer or document specific evaluation infos like gold labels 
         and metrics depicting whether the row matches the gold labels are included, too.
         """
-        df: DataFrame = pd.DataFrame()
 
         if query_labels is None or query_labels.labels is None:
             logger.warning(f"There is no label for query '{query}'. Query will be omitted.")
-            return df
+            return pd.DataFrame()
 
         # remarks for no_answers:
         # Single 'no_answer'-labels are not contained in MultiLabel aggregates.
@@ -450,27 +479,37 @@ class Pipeline(BasePipeline):
         # - the position or offsets within the document the answer was found
         # - the surrounding context of the answer within the document
         # - the gold answers
-        # - the positon or offsets of the gold answer within the document
+        # - the position or offsets of the gold answer within the document
         # - the gold document ids containing the answer
         # - the exact_match metric depicting if the answer exactly matches the gold label
         # - the f1 metric depicting how well the answer overlaps with the gold label on token basis
-        # - the sas metric depciting how well the answer matches the gold label on a semantic basis.
+        # - the sas metric depicting how well the answer matches the gold label on a semantic basis.
         #   this will be calculated on all queries in eval() for performance reasons if a sas model has been provided
-        answers = node_output.get("answers", None)
-        if answers is not None:
-            answer_cols_to_keep = ["answer", "document_id", "offsets_in_document", "context"]
-            df_answers = pd.DataFrame(answers, columns=answer_cols_to_keep)
-            if len(df_answers) > 0:
-                df_answers["type"] = "answer"
-                df_answers["gold_answers"] = [gold_answers] * len(df_answers)
-                df_answers["gold_offsets_in_documents"] = [gold_offsets_in_documents] * len(df_answers)
-                df_answers["gold_document_ids"] = [gold_document_ids] * len(df_answers)
-                df_answers["exact_match"] = df_answers.apply(
-                    lambda row: calculate_em_str_multi(gold_answers, row["answer"]), axis=1)
-                df_answers["f1"] = df_answers.apply(
-                    lambda row: calculate_f1_str_multi(gold_answers, row["answer"]), axis=1)
-                df_answers["rank"] = np.arange(1, len(df_answers)+1)
-                df = pd.concat([df, df_answers])
+
+        partial_dfs = []
+        for field_name in ["answers", "answers_isolated"]:
+            df = pd.DataFrame()
+            answers = node_output.get(field_name, None)
+            if answers is not None:
+                answer_cols_to_keep = ["answer", "document_id", "offsets_in_document", "context"]
+                df_answers = pd.DataFrame(answers, columns=answer_cols_to_keep)
+                if len(df_answers) > 0:
+                    df_answers["type"] = "answer"
+                    df_answers["gold_answers"] = [gold_answers] * len(df_answers)
+                    df_answers["gold_offsets_in_documents"] = [gold_offsets_in_documents] * len(df_answers)
+                    df_answers["gold_document_ids"] = [gold_document_ids] * len(df_answers)
+                    df_answers["exact_match"] = df_answers.apply(
+                        lambda row: calculate_em_str_multi(gold_answers, row["answer"]), axis=1)
+                    df_answers["f1"] = df_answers.apply(
+                        lambda row: calculate_f1_str_multi(gold_answers, row["answer"]), axis=1)
+                    df_answers["rank"] = np.arange(1, len(df_answers)+1)
+                    df = pd.concat([df, df_answers])
+
+            # add general info
+            df["node"] = node_name
+            df["query"] = query
+            df["eval_mode"] = "isolated" if "isolated" in field_name else "integrated"
+            partial_dfs.append(df)
 
         # if node returned documents, include document specific info:
         # - the document_id
@@ -480,33 +519,37 @@ class Pipeline(BasePipeline):
         # - the gold_id_match metric depicting whether one of the gold document ids matches the document
         # - the answer_match metric depicting whether the document contains the answer
         # - the gold_id_or_answer_match metric depicting whether one of the former two conditions are met
-        documents = node_output.get("documents", None)
-        if documents is not None:
-            document_cols_to_keep = ["content", "id"]
-            df_docs = pd.DataFrame(documents, columns=document_cols_to_keep)
-            if len(df_docs) > 0:
-                df_docs = df_docs.rename(columns={"id": "document_id"})
-                df_docs["type"] = "document"
-                df_docs["gold_document_ids"] = [gold_document_ids] * len(df_docs)
-                df_docs["gold_document_contents"] = [gold_document_contents] * len(df_docs)
-                df_docs["gold_id_match"] = df_docs.apply(
-                    lambda row: 1.0 if row["document_id"] in gold_document_ids else 0.0, axis=1)
-                df_docs["answer_match"] = df_docs.apply(
-                    lambda row: 
-                        1.0 if not query_labels.no_answer 
-                            and any(gold_answer in row["content"] for gold_answer in gold_answers) 
-                        else 0.0, 
-                    axis=1)
-                df_docs["gold_id_or_answer_match"] = df_docs.apply(
-                    lambda row: max(row["gold_id_match"], row["answer_match"]), axis=1)
-                df_docs["rank"] = np.arange(1, len(df_docs)+1)
-                df = pd.concat([df, df_docs])
+        for field_name in ["documents", "documents_isolated"]:
+            df = pd.DataFrame()
+            documents = node_output.get(field_name, None)
+            if documents is not None:
+                document_cols_to_keep = ["content", "id"]
+                df_docs = pd.DataFrame(documents, columns=document_cols_to_keep)
+                if len(df_docs) > 0:
+                    df_docs = df_docs.rename(columns={"id": "document_id"})
+                    df_docs["type"] = "document"
+                    df_docs["gold_document_ids"] = [gold_document_ids] * len(df_docs)
+                    df_docs["gold_document_contents"] = [gold_document_contents] * len(df_docs)
+                    df_docs["gold_id_match"] = df_docs.apply(
+                        lambda row: 1.0 if row["document_id"] in gold_document_ids else 0.0, axis=1)
+                    df_docs["answer_match"] = df_docs.apply(
+                        lambda row:
+                            1.0 if not query_labels.no_answer
+                                and any(gold_answer in row["content"] for gold_answer in gold_answers)
+                            else 0.0,
+                        axis=1)
+                    df_docs["gold_id_or_answer_match"] = df_docs.apply(
+                        lambda row: max(row["gold_id_match"], row["answer_match"]), axis=1)
+                    df_docs["rank"] = np.arange(1, len(df_docs)+1)
+                    df = pd.concat([df, df_docs])
 
-        # add general info
-        df["node"] = node_name
-        df["query"] = query
+            # add general info
+            df["node"] = node_name
+            df["query"] = query
+            df["eval_mode"] = "isolated" if "isolated" in field_name else "integrated"
+            partial_dfs.append(df)
 
-        return df
+        return pd.concat(partial_dfs, ignore_index=True)
 
     def get_next_nodes(self, node_id: str, stream_id: str):
         current_node_edges = self.graph.edges(node_id, data=True)
@@ -674,7 +717,12 @@ class Pipeline(BasePipeline):
             component_type = component_instance.pipeline_config["type"]
             component_params = component_instance.pipeline_config["params"]
             components[node] = {"name": node, "type": component_type, "params": {}}
-            component_signature = inspect.signature(type(component_instance)).parameters
+            
+            component_parent_classes = inspect.getmro(type(component_instance))
+            component_signature: dict = {}
+            for component_parent in component_parent_classes:
+                component_signature = {**component_signature, **inspect.signature(component_parent).parameters}
+                
             for key, value in component_params.items():
                 # A parameter for a Component could be another Component. For instance, a Retriever has
                 # the DocumentStore as a parameter.
@@ -737,21 +785,23 @@ class Pipeline(BasePipeline):
 
     def _format_wrong_samples_report(self, eval_result: EvaluationResult, n_wrong_examples: int = 3):
         examples = {
-            node: eval_result.wrong_examples(node, doc_relevance_col="gold_id_or_answer_match", n=n_wrong_examples) 
+            node: eval_result.wrong_examples(node, doc_relevance_col="gold_id_or_answer_match", n=n_wrong_examples)
                 for node in eval_result.node_results.keys()
         }
         examples_formatted = {
-            node: "\n".join([self._format_wrong_sample(example) for example in examples]) 
+            node: "\n".join([self._format_wrong_sample(example) for example in examples])
                 for node, examples in examples.items()
         }
 
         return "\n".join([self._format_wrong_samples_node(node, examples) for node, examples in examples_formatted.items()])
 
-    def _format_pipeline_node(self, node: str, metrics: dict, metrics_top_1):
-        metrics = metrics.get(node, {})
-        metrics_top_1 = {f"{metric}_top_1": value for metric, value in metrics_top_1.get(node, {}).items()}
-        node_metrics = {**metrics, **metrics_top_1}
-        node_metrics_formatted = "\n".join(sorted([f"                        | {metric}: {value:5.3}" for metric, value in node_metrics.items()])) 
+    def _format_pipeline_node(self, node: str, calculated_metrics: dict):
+        node_metrics: dict = {}
+        for metric_mode in calculated_metrics:
+            for metric, value in calculated_metrics[metric_mode].get(node, {}).items():
+                node_metrics[f"{metric}{metric_mode}"] = value
+
+        node_metrics_formatted = "\n".join(sorted([f"                        | {metric}: {value:5.3}" for metric, value in node_metrics.items()]))
         node_metrics_formatted = f"{node_metrics_formatted}\n" if len(node_metrics_formatted) > 0 else ""
         s = (
             f"                      {node}\n"
@@ -761,8 +811,8 @@ class Pipeline(BasePipeline):
         )
         return s
 
-    def _format_pipeline_overview(self, metrics: dict, metrics_top_1: dict):
-        pipeline_overview = "\n".join([self._format_pipeline_node(node, metrics, metrics_top_1) for node in self.graph.nodes])
+    def _format_pipeline_overview(self, calculated_metrics: dict):
+        pipeline_overview = "\n".join([self._format_pipeline_node(node, calculated_metrics) for node in self.graph.nodes])
         s = (
             f"================== Evaluation Report ==================\n"
             f"=======================================================\n"
@@ -789,17 +839,18 @@ class Pipeline(BasePipeline):
         if any(degree > 1 for node, degree in self.graph.out_degree):
             logger.warning("Pipelines with junctions are currently not supported.")
             return
-        
-        metrics_top_n = eval_result.calculate_metrics(doc_relevance_col="gold_id_or_answer_match")
-        metrics_top_1 = eval_result.calculate_metrics(doc_relevance_col="gold_id_or_answer_match", simulated_top_k_reader=1)
+
+        calculated_metrics = {"": eval_result.calculate_metrics(doc_relevance_col="gold_id_or_answer_match"),
+                              "_top_1": eval_result.calculate_metrics(doc_relevance_col="gold_id_or_answer_match", simulated_top_k_reader=1),
+                              " upper bound": eval_result.calculate_metrics(doc_relevance_col="gold_id_or_answer_match", eval_mode="isolated")}
+
         if metrics_filter is not None:
-            metrics_top_n = {node: metrics if node not in metrics_filter 
-                                    else {metric: value for metric, value in metrics.items() if metric in metrics_filter[node]} 
-                                    for node, metrics in metrics_top_n.items()}
-            metrics_top_1 = {node: metrics if node not in metrics_filter 
-                                    else {metric: value for metric, value in metrics.items() if metric in metrics_filter[node]} 
-                                    for node, metrics in metrics_top_1.items()}
-        pipeline_overview = self._format_pipeline_overview(metrics_top_n, metrics_top_1)        
+            for metric_mode in calculated_metrics:
+                calculated_metrics[metric_mode] = {node: metrics if node not in metrics_filter
+                                    else {metric: value for metric, value in metrics.items() if metric in metrics_filter[node]}
+                                    for node, metrics in calculated_metrics[metric_mode].items()}
+
+        pipeline_overview = self._format_pipeline_overview(calculated_metrics)
         wrong_samples_report = self._format_wrong_samples_report(eval_result=eval_result, n_wrong_examples=n_wrong_examples)
 
         print(
