@@ -56,7 +56,7 @@ def test_prediction_attributes(prediction):
 def test_model_download_options():
     # download disabled and model is not cached locally
     with pytest.raises(OSError):
-        impossible_reader = FARMReader("mfeb/albert-xxlarge-v2-squad2", local_files_only=True)
+        impossible_reader = FARMReader("mfeb/albert-xxlarge-v2-squad2", local_files_only=True, num_processes=0)
 
 def test_answer_attributes(prediction):
     # TODO Transformers answer also has meta key
